@@ -20,6 +20,8 @@ pub struct ETree {
     standalone:Option<Vec<u8>>,
     data:Vec<ETreeNode>,
     crlf:String,
+    enable_index:bool,
+    index:HashMap<usize, usize>,
 }
 
 impl ETree {
@@ -47,6 +49,8 @@ impl ETree {
             standalone: None,
             data: Vec::new(),
             crlf: fileformat.to_string(),
+            enable_index: false,
+            index: HashMap::new(),
         };
         out.read(content);
         out.detect_indent();
@@ -57,6 +61,17 @@ impl ETree {
         fs::write(path, self.write())
     }
     #[allow(dead_code)]
+    /// get whether index feature is enabled
+    pub fn get_enable_index(&self) -> bool {
+        self.enable_index
+    }
+    #[allow(dead_code)]
+    /// set whether index feature is enabled (usable for function `pos()`)
+    pub fn set_enable_index(&mut self, enable_index:bool) {
+        self.enable_index = enable_index;
+        self.generate_index();
+    }
+    #[allow(dead_code)]
     /// get XML version
     pub fn get_version(&self) -> Option<String> {
         String::from_utf8(self.version.clone()).ok()
@@ -124,23 +139,39 @@ impl ETree {
         let mut out:Vec<usize> = Vec::new();
         if pos < self.data.len() {
             let route = format!("{}{}#", self.data[pos].get_route(), self.data[pos].get_idx());
-            for i in pos..self.data.len() {
-                if self.data[i].get_route() == route {
+            for i in pos+1..self.data.len() {
+                let curroute = self.data[i].get_route();
+                if curroute == route {
                     out.push(i);
+                } else if !curroute.starts_with(&route) {
+                    break;
                 }
             }
         }
         out
     }
     #[allow(dead_code)]
+    /// get positions of children node with specified name
+    pub fn children_by_name(&self, pos:usize, tagname:&str) -> Vec<usize> {
+        let mut out:Vec<usize> = Vec::new();
+        for i in self.children(pos) {
+            if self.data[i].get_name() == tagname {
+                out.push(i);
+            }
+        }
+        out
+    }
+    #[allow(dead_code)]
     /// get positions of descendant node
     pub fn descendant(&self, pos:usize) -> Vec<usize> {
         let mut out:Vec<usize> = Vec::new();
         if pos < self.data.len() {
             let route = format!("{}{}#", self.data[pos].get_route(), self.data[pos].get_idx());
-            for i in pos..self.data.len() {
+            for i in pos+1..self.data.len() {
                 if self.data[i].get_route().starts_with(&route) {
                     out.push(i);
+                } else {
+                    break;
                 }
             }
         }
@@ -153,11 +184,16 @@ impl ETree {
             None
         } else {
             let mut pos2 = pos;
+            let route = self.data[pos].get_route();
             while pos2 > 0 {
                 pos2 -= 1;
-                if self.data[pos2].get_route() == self.data[pos].get_route() {
+                let curroute = self.data[pos2].get_route();
+                if curroute == route {
                     return Some(pos2);
                 }
+                if !curroute.starts_with(&route) {
+                    break;
+                }
             }
             None
         }
@@ -169,10 +205,15 @@ impl ETree {
             None
         } else {
             let mut pos2 = pos + 1;
+            let route = self.data[pos].get_route();
             while pos2 < self.data.len() {
-                if self.data[pos2].get_route() == self.data[pos].get_route() {
+                let curroute = self.data[pos2].get_route();
+                if curroute == route {
                     return Some(pos2);
                 }
+                if !curroute.starts_with(&route) {
+                    break;
+                }
                 pos2 += 1;
             }
             None
@@ -181,22 +222,26 @@ impl ETree {
     #[allow(dead_code)]
     /// get position by idx
     pub fn pos(&self, idx:usize) -> Option<usize> {
-        for i in 0..self.data.len() {
-            if self.data[i].get_idx() == idx {
-                return Some(i);
+        if self.enable_index {
+            self.index.get(&idx).copied()
+        } else {
+            for i in 0..self.data.len() {
+                if self.data[i].get_idx() == idx {
+                    return Some(i);
+                }
             }
+            None
         }
-        None
     }
     #[allow(dead_code)]
     /// get node by position
-    pub fn node(&self, idx:usize) -> Option<&ETreeNode> {
-        self.data.get(idx)
+    pub fn node(&self, pos:usize) -> Option<&ETreeNode> {
+        self.data.get(pos)
     }
     #[allow(dead_code)]
     /// get mut node by position
-    pub fn node_mut(&mut self, idx:usize) -> Option<&mut ETreeNode> {
-        self.data.get_mut(idx)
+    pub fn node_mut(&mut self, pos:usize) -> Option<&mut ETreeNode> {
+        self.data.get_mut(pos)
     }
     #[allow(dead_code)]
     /// clone a subtree rooted at the node of specified position
@@ -209,6 +254,8 @@ impl ETree {
             standalone: self.standalone.clone(),
             data: Vec::new(),
             crlf: self.crlf.clone(),
+            enable_index: false,
+            index: HashMap::new(),
         };
         let offspring = self.descendant(pos);
         let mut node = self.data[pos].clone();
@@ -231,8 +278,10 @@ impl ETree {
             node.set_idx(self.count);
             node.set_tail(&cell.get_tail());
             node.set_route(&cell.get_route());
-            self.count += 1;
             self.data.insert(cell.get_idx(), node);
+            self.index.insert(self.count, cell.get_idx());
+            self.update_index(cell.get_idx() + 1);
+            self.count += 1;
             Some(cell.get_idx())
         } else {
             None
@@ -247,8 +296,10 @@ impl ETree {
             node.set_idx(self.count);
             node.set_tail(&cell.get_tail());
             node.set_route(&cell.get_route());
-            self.count += 1;
             self.data.insert(cell.get_idx(), node);
+            self.index.insert(self.count, cell.get_idx());
+            self.update_index(cell.get_idx() + 1);
+            self.count += 1;
             Some(cell.get_idx())
         } else {
             None
@@ -263,8 +314,10 @@ impl ETree {
             node.set_idx(self.count);
             node.set_tail(&cell.get_tail());
             node.set_route(&cell.get_route());
-            self.count += 1;
             self.data.insert(cell.get_idx(), node);
+            self.index.insert(self.count, cell.get_idx());
+            self.update_index(cell.get_idx() + 1);
+            self.count += 1;
             Some(cell.get_idx())
         } else {
             None
@@ -289,7 +342,9 @@ impl ETree {
                 let route = format!("{}{}", cell.get_route(), tree.data[i].get_route().get(1..).unwrap());
                 tree.data[i].set_route(&route);
                 self.data.insert(cell.get_idx() + i, tree.data[i].clone());
+                self.index.insert(tree.data[i].get_idx(), cell.get_idx() + i);
             }
+            self.update_index(cell.get_idx() + tree.data.len());
             Some(cell.get_idx())
         } else {
             None
@@ -314,7 +369,9 @@ impl ETree {
                 let route = format!("{}{}", cell.get_route(), tree.data[i].get_route().get(1..).unwrap());
                 tree.data[i].set_route(&route);
                 self.data.insert(cell.get_idx() + i, tree.data[i].clone());
+                self.index.insert(tree.data[i].get_idx(), cell.get_idx() + i);
             }
+            self.update_index(cell.get_idx() + tree.data.len());
             Some(cell.get_idx())
         } else {
             None
@@ -339,7 +396,9 @@ impl ETree {
                 let route = format!("{}{}", cell.get_route(), tree.data[i].get_route().get(1..).unwrap());
                 tree.data[i].set_route(&route);
                 self.data.insert(cell.get_idx() + i, tree.data[i].clone());
+                self.index.insert(tree.data[i].get_idx(), cell.get_idx() + i);
             }
+            self.update_index(cell.get_idx() + tree.data.len());
             Some(cell.get_idx())
         } else {
             None
@@ -366,9 +425,12 @@ impl ETree {
         let mut i = offspring.len();
         while i > 0 {
             i -= 1;
+            self.index.remove(&self.data[offspring[i]].get_idx());
             self.data.remove(offspring[i]);
         }
+        self.index.remove(&self.data[pos].get_idx());
         self.data.remove(pos);
+        self.update_index(pos);
     }
     #[allow(dead_code)]
     /// clear indent and return old indent
@@ -826,6 +888,65 @@ impl ETree {
             }
         }
     }
+    fn generate_index(&mut self) {
+        if self.enable_index {
+            self.index = HashMap::new();
+            for i in 0..self.data.len() {
+                self.index.insert(self.data[i].get_idx(), i);
+            }
+        }
+    }
+    fn update_index(&mut self, pos:usize) {
+        if self.enable_index {
+            for i in pos..self.data.len() {
+                if let Some(x) = self.index.get_mut(&self.data[i].get_idx()) {
+                    *x = i;
+                }
+            }
+        }
+    }
+    #[allow(dead_code)]
+    /// find the first node that matches `path` from the root node
+    pub fn find(&self, path:&str) -> Option<usize> {
+        self.find_at(path, self.root())
+    }
+    #[allow(dead_code)]
+    /// find the first node that matches `path` from specified node
+    pub fn find_at(&self, path:&str, pos:usize) -> Option<usize> {
+        let mut iter = self.find_at_iter(path, pos);
+        iter.next()
+    }
+    #[allow(dead_code)]
+    /// find nodes that matches `path` from the root node
+    pub fn find_iter(&self, path:&str) -> XPathIterator {
+        self.find_at_iter(path, self.root())
+    }
+    #[allow(dead_code)]
+    /// find nodes that matches `path` from specified node
+    pub fn find_at_iter(&self, path:&str, pos:usize) -> XPathIterator {
+        XPathIterator::new(self, path, pos, true)
+    }
+    #[allow(dead_code)]
+    /// find the last node that matches `path` from the root node
+    pub fn rfind(&self, path:&str) -> Option<usize> {
+        self.rfind_at(path, self.root())
+    }
+    #[allow(dead_code)]
+    /// find the last node that matches `path` from specified node
+    pub fn rfind_at(&self, path:&str, pos:usize) -> Option<usize> {
+        let mut iter = self.rfind_at_iter(path, pos);
+        iter.next()
+    }
+    #[allow(dead_code)]
+    /// find nodes in reverse order that matches `path` from the root node
+    pub fn rfind_iter(&self, path:&str) -> XPathIterator {
+        self.rfind_at_iter(path, self.root())
+    }
+    #[allow(dead_code)]
+    /// find nodes in reverse order that matches `path` from specified node
+    pub fn rfind_at_iter(&self, path:&str, pos:usize) -> XPathIterator {
+        XPathIterator::new(self, path, pos, false)
+    }
 }
 
 /// transform root node into a tree
@@ -839,6 +960,8 @@ impl From<ETreeNode> for ETree {
             standalone:None,
             data:Vec::new(),
             crlf:"".to_string(),
+            enable_index: false,
+            index: HashMap::new(),
         };
         node.set_idx(0);
         node.set_route("#");
@@ -846,3 +969,310 @@ impl From<ETreeNode> for ETree {
         tree
     }
 }
+
+/// XPath operation
+///
+/// # Supported syntax:
+/// ## Node query
+/// - `nodename`: the same as `//nodename`
+/// - `*`: any node
+/// - `/`: node in the children of current node
+/// - `//`: node in the descendant of current node
+/// - `.`: current node
+/// - `..`: parent node
+/// - `@attrname`
+/// ## Node Predicate
+/// - `[1]`: first element
+/// - `[last()-1]`: second to last element
+/// - `[position() < 3]`: first and second element
+/// - `[@attrname]`: element with attr `attrname`
+/// - `[@*]`: element with any attr
+/// - `[@attrname='value']`: element with attr `attrname`=`value`
+/// - `[text()='value']`: element which text is equal to `value`
+/// - `[child-tag='value']`: element which contains child `child-tag` and child tag's text is equal to `value`
+/// - `[text()='value' and child-tag='value']`: multiple condition with `and`/`or` and parenthesis
+/// # Search algorithm
+/// 1. `path` is split into multiple parts by consecutive "/".
+///    - e.g. "//tag1/tag2[text()='abc']" is split into ["//tag1", "/tag2[text()='abc']"]
+/// 2. find first part from the specified node
+/// 3. find next part from the result of last find
+/// 4. repeat step 3 until all part finished
+pub struct XPathIterator<'a> {
+    tree: &'a ETree,
+    direction: bool,
+    path_list: Vec<String>,
+    todo_list: Vec<(usize, usize)>,
+}
+
+impl<'a> XPathIterator<'a> {
+    #[allow(dead_code)]
+    fn new(tree:&'a ETree, path:&str, pos:usize, dir:bool) -> Self {
+        let quote = vec!['\'', '"'];
+        let enclose_open = vec!['['];
+        let enclose_close = vec![']'];
+        let mut escaped = false;
+        let mut split_pos = Vec::new();
+        let mut stack1 = Vec::new();
+        let mut stack2 = Vec::new();
+        for item in path.char_indices() {
+            if escaped {
+                escaped = false;
+            } else {
+                if item.1 == '\\' {
+                    escaped = true;
+                } else {
+                    if quote.contains(&item.1) {
+                        if stack1.is_empty() {
+                            stack1.push(item.1);
+                        } else {
+                            if stack1[stack1.len()-1] == item.1 {
+                                stack1.pop();
+                            } else {
+                                stack1.push(item.1);
+                            }
+                        }
+                    } else if stack1.is_empty() {
+                        if enclose_open.contains(&item.1) {
+                            stack2.push(item.1);
+                        } else if enclose_close.contains(&item.1) {
+                            let mut p=0;
+                            while p<enclose_close.len() {
+                                if enclose_close[p] == item.1 {
+                                    break;
+                                }
+                                p+=1;
+                            }
+                            if stack2[stack2.len()-1] == enclose_open[p] {
+                                stack2.pop();
+                            } else {
+                                stack2.push(item.1);
+                            }
+                        } else if item.1 == '/' && stack2.is_empty() {
+                            if split_pos.is_empty() {
+                                split_pos.push(item.0);
+                            } else if split_pos[split_pos.len()-1] + 1 < item.0 {
+                                split_pos.push(item.0);
+                            }
+                        }
+                    }
+                }
+            }
+        }
+        assert!(stack1.is_empty());
+        assert!(stack2.is_empty());
+        let mut path_todo:Vec<String> = Vec::new();
+        if split_pos.is_empty() {
+            path_todo.push(path.to_string());
+        } else {
+            let mut pos1 = 0;
+            let mut posidx = 0;
+            while posidx < split_pos.len() {
+                path_todo.push(path.get(pos1..split_pos[posidx]).unwrap().to_string());
+                pos1 = split_pos[posidx];
+                posidx += 1;
+            }
+            path_todo.push(path.get(pos1..).unwrap().to_string());
+        }
+        if path_todo[0] == "" || path_todo[0] == "." {
+            path_todo.remove(0);
+        } else if path_todo[0] == "/" {
+        } else if path_todo[0] == ".." {
+            path_todo.remove(0);
+            path_todo.insert(0, "/..".to_string());
+        } else {
+            let element = path_todo.remove(0);
+            path_todo.insert(0, format!("//{}", element));
+        }
+        Self {
+            tree: tree,
+            direction: dir,
+            path_list: path_todo,
+            todo_list: vec![(pos, 0)],
+        }
+    }
+    fn _find(&self, path:&str, pos:usize) -> Vec<usize> {
+        let mut result:Vec<usize> = Vec::new();
+        if path == "/." {
+            result.push(pos);
+        } else if path == "/.." {
+            if let Some(parent) = self.tree.parent(pos) {
+                result.push(parent);
+            }
+        } else {
+            let re = Regex::new(r"^(/+)(.+)$").unwrap();
+            if let Some(c) = re.captures(path) {
+                let m1 = c.get(1).unwrap().as_str();
+                let m2 = c.get(2).unwrap().as_str();
+                let container = if m1 == "//" {
+                    self.tree.descendant(pos)
+                } else { /* "/" */
+                    self.tree.children(pos)
+                };
+                if m2.starts_with("@") {
+                    let attr = m2.get(1..).unwrap();
+                    if attr == "*" {
+                        for positem in container {
+                            if self.tree.node(positem).unwrap().get_attr_count() > 0 {
+                                result.push(positem);
+                            }
+                        }
+                    } else {
+                        for positem in container {
+                            if self.tree.node(positem).unwrap().get_attr(attr).is_some() {
+                                result.push(positem);
+                            }
+                        }
+                    }
+                } else {
+                    let re = Regex::new(r"^(.+?)(?:\[(.+?)\])?$").unwrap();
+                    if let Some(c) = re.captures(m2) {
+                        let tag = c.get(1).unwrap().as_str();
+                        let mut container:Vec<usize> = container.iter().filter(|&x| self.tree.node(*x).unwrap().get_name()==tag).map(|x| *x).collect();
+                        if let Some(predicate) = c.get(2) {
+                            let pat1 = Regex::new(r"\band\b").unwrap();
+                            let pat2 = Regex::new(r"\bor\b").unwrap();
+                            let expr = pat2.replace_all(pat1.replace_all(predicate.as_str(), "&&").into_owned().as_str(), "||").into_owned();
+                            let expr = expr.replace("=", "==").replace("!==", "!=").replace(">==", ">=").replace("<==", "<=");
+                            let re = Regex::new(r"((?P<attr>@\S+?)|(?P<func>\S+?\s*\(\s*\))|(?P<tag>\S+?))\s*=").unwrap();
+                            let mut params_attr:Vec<String> = Vec::new();
+                            let mut params_func:Vec<String> = Vec::new();
+                            let mut params_tag:Vec<String> = Vec::new();
+                            for param in re.captures_iter(&expr) {
+                                if param.name("attr").is_some() {
+                                    let x = param.name("attr").unwrap().as_str().to_string();
+                                    if !params_attr.contains(&x) {
+                                        params_attr.push(x);
+                                    }
+                                } else if param.name("func").is_some() {
+                                    let x = param.name("func").unwrap().as_str().to_string();
+                                    if !params_func.contains(&x) {
+                                        params_func.push(x);
+                                    }
+                                } else if param.name("tag").is_some() {
+                                    let x = param.name("tag").unwrap().as_str().to_string();
+                                    if !params_tag.contains(&x) {
+                                        params_tag.push(x);
+                                    }
+                                }
+                            }
+                            let container_len = container.len();
+                            for i in 0..container_len {
+                                let mut found = true;
+                                let mut cur_expr = expr.clone();
+                                for param in params_attr.iter() {
+                                    if let Some(v) = self.tree.node(container[i]).unwrap().get_attr(param.get(1..).unwrap()) {
+                                        cur_expr = cur_expr.replace(param.as_str(), format!("'{}'", v).as_str());
+                                    } else {
+                                        found = false;
+                                        break;
+                                    }
+                                }
+                                if !found {
+                                    break;
+                                }
+                                for param in params_func.iter() {
+                                    if param.starts_with("text") {
+                                        cur_expr = cur_expr.replace(param.as_str(), format!("'{}'", self.tree.node(container[i]).unwrap().get_text().unwrap_or("".to_string())).as_str());
+                                    } else if param.starts_with("position") {
+                                        cur_expr = cur_expr.replace(param.as_str(), format!("{}", i+1).as_str());
+                                    } else if param.starts_with("last") {
+                                        cur_expr = cur_expr.replace(param.as_str(), format!("{}", container_len).as_str());
+                                    }
+                                }
+                                if params_tag.len() > 0 {
+                                    let mut subfound:Vec<Vec<usize>> = Vec::new();
+                                    let mut curcomb:Vec<usize> = Vec::new();
+                                    for _ in 0..params_tag.len() {
+                                        subfound.push(Vec::new());
+                                        curcomb.push(0);
+                                    }
+                                    let subchildren = self.tree.children(container[i]);
+                                    for subi in subchildren {
+                                        for subj in 0..params_tag.len() {
+                                            if self.tree.node(subi).unwrap().get_name() == params_tag[subj] {
+                                                subfound[subj].push(subi);
+                                            }
+                                        }
+                                    }
+                                    if subfound.iter().all(|ref x| x.len() > 0) {
+                                        let backup_expr = cur_expr;
+                                        let mut exit_flag = false;
+                                        loop {
+                                            cur_expr = backup_expr.clone();
+                                            for subj in 0..params_tag.len() {
+                                                cur_expr = cur_expr.replace(params_tag[subj].as_str(),
+                                                    format!("'{}'",
+                                                        self.tree.node(subfound[subj][curcomb[subj]]).unwrap().get_text().unwrap_or("".to_string())).as_str());
+                                            }
+                                            if eval::eval(cur_expr.as_str()) == Ok(eval::to_value(true)) {
+                                                result.push(container[i]);
+                                                break;
+                                            }
+                                            let mut subi = curcomb.len() - 1;
+                                            loop {
+                                                curcomb[subi] += 1;
+                                                if curcomb[subi] >= subfound[subi].len() {
+                                                    curcomb[subi] = 0;
+                                                    if subi > 0 {
+                                                        subi -= 1;
+                                                    } else {
+                                                        exit_flag = true;
+                                                        break;
+                                                    }
+                                                } else {
+                                                    break;
+                                                }
+                                            }
+                                            if exit_flag {
+                                                break;
+                                            }
+                                        }
+                                    }
+                                } else {
+                                    if eval::eval(cur_expr.as_str()) == Ok(eval::to_value(true)) {
+                                        result.push(container[i]);
+                                    }
+                                }
+                            }
+                        } else {
+                            result.append(&mut container);
+                        }
+                    } else {
+                        // Syntax error
+                    }
+                }
+            } else {
+                // Syntax error
+            }
+        }
+        result
+    }
+}
+
+impl<'a> Iterator for XPathIterator<'a> {
+    type Item = usize;
+    fn next(&mut self) -> Option<Self::Item> {
+        while !self.todo_list.is_empty() {
+            let item = self.todo_list.pop().unwrap();
+            if item.1 >= self.path_list.len() {
+                return Some(item.0);
+            } else {
+                let result = self._find(&self.path_list[item.1], item.0);
+                let rlen = result.len();
+                let mut ridx = rlen;
+                if self.direction {
+                    while ridx > 0 {
+                        ridx -= 1;
+                        self.todo_list.push((result[ridx], item.1+1));
+                    }
+                } else {
+                    while ridx > 0 {
+                        ridx -= 1;
+                        self.todo_list.push((result[rlen - ridx - 1], item.1+1));
+                    }
+                }
+            }
+        }
+        None
+    }
+}
