@@ -4,8 +4,6 @@
 
 mod etreenode;
 mod etree;
-mod xpath;
 
 pub use self::etreenode::ETreeNode;
 pub use self::etree::ETree;
-pub use self::xpath::XPath;
